<copyright> gtomk Scribble example.
    A rewrite in TOM of the original scribble.c example,
    as it appears in the gtk+-0.99.9 distribution.
    By <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.

    This file is part of GTOMK.  GTOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GTOMK distribution; see the file COPYING.LIB.

    <id>$Id: Scribble.t,v 1.7 1998/07/22 12:50:48 yacc Exp $</id>
    </copyright>

implementation class
Scribble: Constants
{
  <doc> The backing pixmap for the drawing area.  </doc>
  static GdkPixmap pixmap;
}

<doc> Create a new backing pixmap of the appropriate size.  </doc>
boolean
  configure_event (GtkWidget, pointer) (widget, event)
{
  if (pixmap != nil)
    [pixmap dispose];

  int w, h;
  (w, h) = [widget allocationSize];

  pixmap = [GdkPixmap new ([widget window], w, h, -1)];
  [pixmap rectangle ([[widget style] white_gc], TRUE, 0, 0, w, h)];

  = TRUE;
}

<doc> Redraw the the exposed area of the drawing {area} from the backing
    {pixmap}.  </doc>
boolean
  expose_event (GtkDrawingArea, pointer) (area, event)
{
  int x, y, w, h;
  (x, y, w, h) = [EventExpose area event];

  [[area window] pixmap ([[area style] fg_gc [area state]],
			 pixmap, x, y, x, y, w, h)];
}

protected void
  draw_brush_at (int, int) (x, y)
	     in GtkWidget widget
{
  int px = x - 5, py = y - 5, w = 10, h = 10;

  [pixmap rectangle ([[widget style] black_gc], TRUE, px, py, w, h)];
  [widget draw (px, py, w, h)];
}
  
boolean
  button_press_event (GtkDrawingArea, pointer) (area, event)
{
  if ([EventButton button event] == 1 && pixmap != nil)
    [self draw_brush_at [EventButton position event] in area];

  = TRUE;
}

boolean
  motion_notify_event (GtkDrawingArea, pointer) (area, event)
{
  int x, y, state;

  if ([EventMotion is_hint event])
    (, x, y, state) = [[area window] get_pointer];
  else
    {
      (x, y) = [EventMotion position event];
      state = [EventMotion state event];
    }

  if (state & GDK_BUTTON1_MASK != 0 && pixmap != nil)
    [self draw_brush_at (x, y) in area];

  = TRUE;
}

void
  quit
{
  [gtomk exit 0];
}

int
  main MutableArray args
{
  [gtomk init args];

  GtkWindow window = [GtkWindow new GTK_WINDOW_TOPLEVEL];
  [window set_title "Test input!"];

  GtkVBox vbox = [GtkVBox new (FALSE, 0)];
  [window add vbox];
  [vbox show];

  [window connect "destroy" to self using selector (void quit) after: NO];

  GtkDrawingArea drawing_area = [GtkDrawingArea new];
  [drawing_area size (200, 200)];
  [vbox pack_start (drawing_area, TRUE, TRUE, 0)];
  [drawing_area show];

  [drawing_area connect "expose_event" to self
		using selector (boolean expose_event (GtkDrawingArea, pointer))
		includeSender: YES];
  [drawing_area connect "configure_event" to self
		using selector (boolean configure_event (GtkWidget, pointer))
		includeSender: YES];
  [drawing_area connect "motion_notify_event" to self
		using selector (boolean motion_notify_event (GtkDrawingArea, pointer))
		includeSender: YES];
  [drawing_area connect "button_press_event" to self
		using selector (boolean button_press_event (GtkDrawingArea, pointer))
		includeSender: YES];

  [drawing_area set_events (GDK_EXPOSURE_MASK | GDK_LEAVE_NOTIFY_MASK
			    | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK
			    | GDK_POINTER_MOTION_HINT_MASK)];

  GtkButton button = [GtkButton new_with_label "Quit"];
  [vbox pack_start (button, FALSE, FALSE, 0)];
  [button connect "clicked" to window using selector (void destroy)];

  [button show];
  [window show];
  [gtomk main];
}

end;

implementation instance
Scribble

end;
