<copyright> gdomk Event class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.

    This file is part of GDOMK.  GDOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GDOMK distribution; see the file COPYING.LIB.

    <id>$Id: Event.t,v 1.4 1998/07/16 13:05:21 yacc Exp $</id>
    </copyright>

<c>
#include "glue.h"
</c>

implementation class
GdkEvent: GdkWrapper

<doc> Return the event type of the given GdkEvent.  </doc>
int (type)
  eventType pointer p
{
<c>
  type = ((GdkEvent *) p)->type;
</c>
}

end;

implementation instance
GdkEvent

<doc> Return the event type of the receiving event.  </doc>
int
  eventType
{
  = [isa eventType object];
}

end;

/******************** EventButton ********************/

implementation class
EventButton: GdkEvent

<doc> Only allow GDK_BUTTON_PRESS events to be wrapped by an EventExpose.  </doc>
instance (id)
  wrap pointer object
pre
  [self eventType object] == GDK_BUTTON_PRESS
{
  = [super wrap object];
}

<doc> Return the button pressed in the {GdkEventButton} {event}.  </doc>
int (button)
  button pointer event
pre
  [self eventType event] == GDK_BUTTON_PRESS
{
<c>
  button = ((GdkEventButton *) event)->button;
</c>
}

<doc> Return the position in the {GdkEventButton} {event}.  </doc>
(int, int) (x, y)
  position pointer event
pre
  [self eventType event] == GDK_BUTTON_PRESS
{
<c>
  x = ((GdkEventButton *) event)->x;
  y = ((GdkEventButton *) event)->y;
</c>
}

end;

implementation instance
EventButton

int
  button
{
  = [isa button object];
}

(int, int)
  position
{
  = [isa position object];
}

end;

/******************** EventExpose ********************/

implementation class
EventExpose: GdkEvent

<doc> Only allow GDK_EXPOSE events to be wrapped by an EventExpose.  </doc>
instance (id)
  wrap pointer object
pre
  [self eventType object] == GDK_EXPOSE
{
  = [super wrap object];
}

<doc> Return the area in the {GdkEventExpose} {event}.  </doc>
(int, int, int, int) (x, y, w, h)
  area pointer event
pre
  [self eventType event] == GDK_EXPOSE
{
<c>
  x = ((GdkEventExpose *) event)->area.x;
  y = ((GdkEventExpose *) event)->area.y;
  w = ((GdkEventExpose *) event)->area.width;
  h = ((GdkEventExpose *) event)->area.height;
</c>
}

end;

implementation instance
EventExpose

end;

/******************** EventMotion ********************/

implementation class
EventMotion: GdkEvent

<doc> Only allow GDK_MOTION_NOTIFY events to be wrapped by an EventMotion.
    </doc>
instance (id)
  wrap pointer object
pre
  [self eventType object] == GDK_MOTION_NOTIFY
{
  = [super wrap object];
}

boolean (is_hint)
  is_hint pointer event
pre
  [self eventType event] == GDK_MOTION_NOTIFY
{
<c>
  is_hint = !!((GdkEventMotion *) event)->is_hint;
</c>
}

<doc> Return the position in the {GdkEventMotion} {event}.  </doc>
(int, int) (x, y)
  position pointer event
pre
  [self eventType event] == GDK_MOTION_NOTIFY
{
<c>
  x = ((GdkEventMotion *) event)->x;
  y = ((GdkEventMotion *) event)->y;
</c>
}

<doc> Return the state in the {GdkEventMotion} {event}.  </doc>
int (state)
  state pointer event
pre
  [self eventType event] == GDK_MOTION_NOTIFY
{
<c>
  state = ((GdkEventMotion *) event)->state;
</c>
}

end;

implementation instance
EventMotion

end;
