<copyright> gdomk Wrapper class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.

    This file is part of GDOMK.  GDOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GDOMK distribution; see the file COPYING.LIB.

    <id>$Id: Wrapper.t,v 1.4 1998/04/09 09:42:05 tiggr Exp $</id>
    </copyright>

<c>
#include "glue.h"
</c>

<doc> Generic struct wrapper.  </doc>
implementation class
GdkWrapper: gdomk, Constants
{
  <doc> Mapping from all (Gdk) wrappers to TOM wrappers.  </doc>
  static MutablePointerDictionary all_gdk_wrappers;
}

<doc> Initialize the {all_gdk_wrappers}.  </doc>
void
  load Array arguments
{
  all_gdk_wrappers = [MutablePointerDictionary new];
  [all_gdk_wrappers setIsContainer YES];
}

<doc> Return the TOM wrapper standing for the argument {object}.  The
    proxy is created if desired.  </doc>
instance (id) (tom_wrapper)
    proxy pointer object
  create: boolean create_p = NO
pre
  create_p -> self != [GdkWrapper self]
post
  tom_wrapper != nil
{
  tom_wrapper = all_gdk_wrappers[object];
  if (!tom_wrapper)
    {
      // SELF need not be the right class...
      // Sun Apr  5 18:30:25 1998, tiggr@gerbil.org
      tom_wrapper = [self wrap object];
    }
}

// There should be a precondition stating that !all_gdk_wrappers[object]
// but that would make subclasses' preconditions _weakening_.  So much
// for Eiffel's precondition semantics, as mimiced by TOM :-(
// Thu Apr  2 15:21:31 1998, tiggr@gerbil.org
<doc> Return a new wrapper for the {object}.  The default implementation
    simply does a {alloc} {init pointer}.  Subclasses could do something
    nasty with reference counts.  </doc>
instance (id)
  wrap pointer object
{
  = [[self alloc] init object];
}

end;

implementation instance
GdkWrapper
{
  <doc> The underlying (gdk) object.  </doc>
  pointer object;
}

<doc> Designated initializer.  </doc>
protected id
  init pointer obj
pre
  !all_gdk_wrappers[obj]
{
  = [super init];

  object = obj;
  all_gdk_wrappers[obj] = self;
}

<doc> Remove the receiving wrapper from {all_gdk_wrappers}, and set the
    {object} to the null pointer.  Does nothing if the {object} already is
    the null pointer.  </doc>
void
  dealloc
{
  if (!!object)
    {
      [all_gdk_wrappers remove object];
      object = ({pointer null;});
    }
}

<doc> Dispose of the receiving object.  This simply invoked {dealloc}, if
    that has not already been done so.  </doc>
void
  dispose
{
  if (!!object)
    [self dealloc];
}

end;
