<copyright> gtomk Object class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.

    This file is part of GTOMK.  GTOMK is distributed under the terms of
    the GNU Library General Public License, a copy of which can be found
    in the GTOMK distribution; see the file COPYING.LIB.

    <id>$Id: Object.t,v 1.13 1999/02/15 21:57:33 tiggr Exp $</id>
    </copyright>

<c>
#include "glue.h"
</c>

implementation class
GtkObject: State, Conditions, GenericWrapper
{
  /* Make this local when thread-safety is needed.
   * Fri Apr  3 22:54:46 1998, tiggr@gerbil.org */
  <doc> Buffer used to construct class names.  </doc>
  static MutableByteString class_name_buffer;
}

void
  load Array arguments
{
  class_name_buffer = [MutableByteString withCapacity 32];
}

<doc> Return the TOM object standing for the argument {gtk_object}.
    The proxy is created if {create_p}.  </doc>
instance (id) (object)
    proxy pointer gtk_object
  create: boolean create_p = NO
post
  object != nil
{
<c>
  object = gtk_object_get_data (gtk_object, "tom_id");
</c>

  if (!object && create_p)
    {
      [class_name_buffer truncate 0];
<c>
      {
	DECL_USEL (tom, i_writeBytes_i_from_p);
	char *s = gtk_type_name (GTK_OBJECT_TYPE (GTK_OBJECT (gtk_object)));
	TRT_SEND (_II_, c_gtomk_GtkObject_class_name_buffer,
		  USEL (tom, i_writeBytes_i_from_p), strlen (s), s);
      }
</c>
      class (State) the_class = [Runtime classNamed class_name_buffer];
      if (!the_class)
	the_class = [[Condition for class_name_buffer
				class unknown-class-condition
				message "unknown class"] signal];
      object = [id (the_class) wrap gtk_object];
    }
}

<doc> Return a new wrapper for the {object}.  The default implementation
    simply does a {alloc} {init pointer}.  Subclasses could do something
    more nasty.  </doc>
instance (id)
  wrap pointer object
{
  = [[self alloc] init object];
}

pointer 
  ptrvalue instance (id) object
{
  = [object gtkObject];
}

end;

implementation instance
GtkObject
{
  <doc> The underlying gtk object.  </doc>
  pointer object;

  <doc> The id of the handler registered to the destroy signal of the
      {object}.  Needed to disconnect it when this TOM object becomes
      garbage before the Gtk object does.  </doc>
  int destroy_handler_id;
}

<doc> Return the {gtkObject} for which this object stands.  </doc>
pointer
  gtkObject
{
  = object;
}

<doc> Designated initializer.  </doc>
protected id
  init pointer gtk_object
{
  = [super init];

  object = gtk_object;
<c>
  gtk_object_set_data (gtk_object, "tom_id", self);
</c>

  /* Be informed of destruction of the OBJECT.  */
  destroy_handler_id = [self connect "destroy" to self
			     using selector (void destroy_proxy pointer)
			     after: NO includeSender: YES];
}

<doc> The designated initializer of {super}.  Do not allow this.  </doc>
id
  init
pre
  FALSE
{
  [self unimplemented cmd
	message: "init called (search for missing implementation of new"];
}

<doc> Unset the "tom_id" the receiving object has with its Gtk-{object}.
    If {disconnect_p}, disconnect the destruction handler as well.  </doc>
void
  detach boolean and_disconnect
pre
  !object == FALSE
{
  pointer gtk_object = object;
<c>
  gtk_object_remove_data (gtk_object, "tom_id");
</c>
  if (and_disconnect)
    [self disconnect destroy_handler_id];

  object = ({pointer null;});
}

<doc> Be informed of the destruction of the {gtk_object}, which supposedly
    is our {object}.  </doc>
void
  destroy_proxy pointer gtk_object
pre
  !object || object == gtk_object
{
  if (!object == FALSE)
    [self detach FALSE];
}

<doc> Register invocation of the {sel} to the {receiver} for occurence of
    the signal {signal_name} on the {object} of the receiving {Object}.

    If {include_sender} is {TRUE}, the first argument to the signal method
    will be the sending object, and the number of arguments to the
    selector must thus be one more than the number of signal arguments.
    If the first argument to the selector is a {pointer}, a pointer to the
    underlying Gtk object will be passed.  If it is a reference, a TOM
    proxy object will be passed.

    If {allow_mismatch} is {TRUE}, the number of arguments to the selector
    is allowed to be less than the number of arguments provided by the
    signal.  Furthermore, the return type of the selector need not match
    the expected type of the signal, in which case 0 (or 0.0 or NULL) will
    be returned.  </doc>
int (handler_id)
	     connect String signal_name
		  to All receiver
	       using selector sel
	      after: boolean after = TRUE
      includeSender: boolean include_sender = FALSE
  overrideArguments: boolean allow_mismatch = FALSE
{
  UniqueString us = [UniqueString with signal_name];
  pointer obj = object;
  int len;

<c>
  {
    GtkSignalQuery *siginfo;
    char *signame;
    gtomk_glue *cb;

    C_STRING_WITH_TOM_STRING (signame, len, us);
    siginfo = gtk_signal_query (gtk_signal_lookup (signame,
						   GTK (obj)->klass->type));
    if (!siginfo)
      {
</c>
	[[Condition for self class program-condition
		    message [[MutableByteString new]
			      print ("no such signal: `", signal_name, "'")]]
	  signal];
	return; 
<c>
      }

    cb = g_new (__typeof__ (*cb), 1);
    cb->prefix_sender = include_sender;
    cb->allow_mismatch = allow_mismatch;
    cb->rcv = (void *) receiver;
    cb->sel = sel;

    handler_id
      = gtk_signal_connect_full (obj, signame, NULL,
				 (GtkCallbackMarshal) gtomk_signal_relay,
				 cb, gtomk_free, 0, after);
  }
</c>
}

<doc> Remove the signal handler that is connected to the receiving object
    with a handler identified by the {handler_id}.  </doc>
void
  disconnect int handler_id
{
  pointer obj = object;
<c>
  gtk_signal_disconnect (obj, handler_id);
</c>
}

<doc> Detach us from the Gtk {object} if it is not {nil} yet.  </doc>
void
  dealloc
{
  if (!object == FALSE)
    [self detach YES];
}

<doc> Dispose of this object.  Employs {detach}.  </doc>
void
  dispose
{
  [self detach YES];
}

/******************** GtkObject functions ********************/

<doc> Invoke {gtk_object_destroy}.  </doc>
void
  destroy
{
  pointer obj = object;
<c>
  gtk_object_destroy (obj);
</c>
}

<doc> Invoke {gtk_object_unref} on the {object}.  </doc>
void
  unref
{
  pointer obj = object;
<c>
  if (obj)
    gtk_object_unref (obj);
</c>
}

<doc> Invoke {gtk_object_ref} on the {object}.  </doc>
void
  ref
{
  pointer obj = object;
<c>
  if (obj)
    gtk_object_ref (obj);
</c>
}

<doc> stores a given {GtkObjectDataItem} under a given key in the object.
</doc>

void
  set_data String key
  data GtkObjectDataItem data 
{
  pointer go=object;
  
  <c> {
    int len;
    char *ckey;
    DECL_USEL (tom, _pi__byteStringContents);
    C_STRING_WITH_TOM_STRING (ckey, len, key);
    
    gtk_object_set_data ((GtkObject *) go, ckey, (gpointer) data); 
  } </c>
}

<doc> Retrieve a {GtkObjectDataItem} with the given {key}.  See the
      docs on {GtkObjectDataItem} for caveats of usage.  </doc>

Any (res)
  get_data String key
{
  pointer go=object;
  GtkObjectDataItem di;
  <c> {
    int len;
    char *ckey;
    DECL_USEL (tom, _pi__byteStringContents);
    C_STRING_WITH_TOM_STRING (ckey, len, key);

    di=(i_gtomk_GtkObjectDataItem)
         gtk_object_get_data ((GtkObject *) go, ckey); 
  } </c>
  res=[di dataitem];
}

end;

<doc> Stores a keyed DataItem in a {GtkObject}. 
      Should it be collected, it deletes itself out of the GtkObject,
      not doing so may result in random crashes.
</doc>

implementation class
GtkObjectDataItem : State
end;

implementation instance 
GtkObjectDataItem 
{
  <doc> stores the object we are stored in. </doc>
  GtkObject storedAt;
  <doc> stores the key under which we are stored. </doc>
  String key;
}

<doc> store us in the given object under the given key. </doc>

id
  initForObject GtkObject obj
            key String k
{
  storedAt=obj;
  key=[k copy];
  [obj set_data key data self];
}

<doc> initialise a nonfunct copy. </doc>

id
  init
{
  storedAt=nil;
  key=nil;
}

<doc> {GtkObject} get_data returns what this method returns. </doc>

Any 
  dataitem
{
  = Any(self);
}

void
  dealloc
{
  if ( (storedAt!=nil) &&
       (key!=nil) ) {
    [storedAt set_data key data nil];
  }
}
end;

<doc> This is an alternative dataitem for {GtkObject}. For it function proper,
      it is enough to store it in the linked object, while in case of
      {GtkObjectDataItem} you need to be sure, that dealloc will be called by
      any children properly
.
</doc>

implementation class
GtkObjectLinkedItem : GtkObjectDataItem 
end;

implementation instance
GtkObjectLinkedItem 
{
  <doc> stores the linked Item that is returned by {GtkObject}s get_data
        method. </doc>
  Any linkeditem;
}

id
  initForObject GtkObject obj
            key String k
     LinkedItem All item
{
  storedAt=obj;
  key=[k copy];
  linkeditem=Any(item);
  [obj set_data key data self];
}

Any
  dataitem 
{
  = linkeditem;
}

end;
